#!/bin/tclsh
#--------------------------------------------------------
#
# movie.tcl 2.0.3a (iMDb grabber) by UKP and GOD-EMPEROR
#
# Info: the only argument this script takes is the nfo
#   path/filename, parses it for the imdb url, then
#   using web.tcl it grabs the info from imdb and adds
#   it to glftpd.log
#
#--------------------------------------------------------
# Changes:
#  2.0.1 updated for new web.tcl 1.06m
#  2.0.2 file output option for imdb data - contrib. kentaur
#  2.0.3 support for German imdb urls - contrib. stranger7
#
#--------------------------------------------------------
# Requirements:
#  to resolve url names, libnss_* and libresolv* must
#  be copied to glftpd/lib, also copy resolv.conf and
#  to glftpd/etc, and 'services' if you run dns locally
# 
#--------------------------------------------------------
#
#  movie.log will record all the web activity for url
#  requests, read it if theres any problems...
#
set httplog "/sitestat/movie.log"

#--------------------------------------------------------
#
# Getnfo provided a feature to write certain imdb info
# to a file within the uploaded nfo's dir, this is that
# feature, specify the filename, or set blank to disable
#
set datafile ".imdbdata"


################################################ begin code.
set gllog "/ftp-data/logs/glftpd.log"

set ff [open $httplog a+]
proc putlog msg {
 global ff
 puts $ff $msg
}
source /bin/web.tcl

#--------------------------------------------------------
#
# to disable http activity logging, turn off debug below
#
#---------------------------

set web::debug 1
set web::adaptive 1
set web::htbuffer(chunking) 0

#---------------------------
proc dfout {file ow gr line} {
  catch {open $file w} fd
  if [string match "file*" $fd] {
    catch {puts $fd $line} boo
    close $fd
    file attributes $file -owner $ow -group $gr
  }
}

proc init {File} {
  global gllog datafile
  set ext [string range $File end-2 end]
  if ![string match {[nN][fF][oO]} $ext] {exit 0}
  set dir [pwd]
        
## search the nfo for a valid imdb link
  set fh [open $File r] ;  set url ""
  set lines [read $fh [file size $File]]
  close $fh
  set remap "{\r} { } {\[} { } {\]} { } {\{} { } {\}} { }"
  foreach txt [split $lines \n] {
    if [regexp {\w+.[iI][mM][dD][bB].[cC][oO][mM]/Title\?\d{5,8}} [string map $remap $txt] url] {break}
  }
  if {[string length $url] == 0} {exit 0}

### url found, lets go grab it
  foreach {host page} [split $url "/"] {set url "http://$url" ; break}
  set html [web::htopen GET $host:80 /$page]
  if {$web::iware != "complete"} {
    putlog "\nERROR failed to retrieve the page request ($web::iware) - retrying..\n"
    set html [web::htopen GET $host:80 /$page]
  }
  set boo 0
## some defaults
  set title "unavailable"
  set rating "0 (none)"
  set director "John Doe"
  set length "N/A"
  set genre "Unknown"
  set plot "no summary"
  set mpaa "..."
## get title
  if [regexp {<title>[^<]+} $html title] {
        set pos [expr [string last > $title] + 1]
        set title [string range $title $pos end]
  }
## get iMDb rating
  if [regexp {<b>\d.\d/10</b> \([\w\s\d,]+\)} $html rating] {
	set pos [string last ( $rating]
	set pos1 [string first { } $rating $pos]
	incr pos ; incr pos1 -1
        set rating "[string range $rating 3 5] ([string range $rating $pos $pos1])"
  }
## get director
  if [regexp {(Directed by|Regie)</b><br>\n<[^>]+>[^<]+} $html director] {
        set pos [string last > $director] ; incr pos
        set director [string range $director $pos end]
  }
## get genre
  if [regexp {<b class=\"ch\">Genre:</b>[\s\n]?<[^>]+>[\w]+} $html genre] {
        set pos [string last > $genre] ; incr pos
        set genre [string range $genre $pos end]
  }
## get plot
  if [regexp {<b class=\"ch\">(Plot (Summary|Outline)|Kurzbeschreibung|Inhaltsangabe):</b>[\s\n]+[^<]+} $html plot] {
        set pos [string last > $plot] ; incr pos
        set plot [string range $plot $pos end]
        regsub -all {[\n\s]+} [string map "&#34; {} {\"} '" $plot] { } plot
	set plot [string trim $plot]
  }
## get runtime
  if [regexp {<b class=\"ch\">(Runtime|L\&auml;nge):</b>([\n\s]+)([\w:]+)(\d+)} $html length] {
	regsub -all {[\n\s]+} $length {} length
	set pos [string last > $length] ; incr pos
        set length [string range $length $pos end]
	set pos [string last : $length]
	if {$pos != -1} {incr pos ; set length [string range $length $pos end]}
	append length "min"
  }

### LOG our IMDB line.
  set datetime [clock format [clock seconds] -format "%a %b %d %H:%M:%S %Y"]
  set fp [open $gllog a]
#--------------------------------------------------------
# valid variables are:
#  dir, url, rating, director, genre, plot, length, title
#
# note: mpaa rating no longer available
#--------------------------------------------------------
  puts $fp "$datetime IMDB: \"$dir\" \"$url\" \"$length $rating\" \"$genre\" \"$plot\" \"$title\" \"$director\""
  close $fp
  if {$datafile != ""} {
    set line "Title:...: \"$title\"\n"
 append line "Rating...: \"$rating\"\n"
 append line "Length...: \"$length\"\n"
 append line "Genre....: \"$genre\"\n"
 append line "Director.: \"$director\"\n"
 append line "Plot.....: \"$plot\"\n"
 append line "URL:.....: \"$url\"\n\n"
 append line "movie2.tcl 2.0.3a"
    dfout $datafile [file attributes $dir -owner] [file attributes $dir -group] $line
  }
}

init [lindex $argv 0]
close $ff
exit 0
