#!/bin/tclsh
#
#  Sitestat PRE (rev 36) for Sitestat 5.x by GOD-EMPEROR
#     - developed from v_pre.tcl
#
#--------------------------------------
# stick this into glftpd's bin dir and 'chmod +s /bin/tclsh'
#
# recommended setup: add the following to your glftpd.conf:
#
# site_cmd EDITPRE  EXEC  /bin/ss-pre.tcl
# custom-editpre    1
# site_cmd PRE      EXEC  /bin/ss-pre.tcl pre
# custom-pre        !8 *
#
#--------------------------------------
# big thanks to vman who's v_pre.tcl this script used as a framework,
# even if the code has been almost entirely rewritten it saved some
# time and effort...
#
# features:
# - dated dir support without requiring a today link
# - handles preing to same and seperate drives
# - allup stats updating upon pre, toggle
# - crediting file owners and/or pre'er upon pre
#

#--------------------------------------
# path/filename to ss-pre.cfg 
#
set tcfg "/bin/ss-pre.cfg"

#--------------------------------------
# enable imdb support (movie2.tcl)
#
set mvs 1

#--------------------------------------
# user crediting options:
#  0 -> no credit given
#  1 -> give credit to owners after pre
#  2 -> give pre'er all the credit
#
set userup 1

#--------------------------------------
# stats options:
#  0 -> no stats updated
#  1 -> add files to allup
#
set statup 0

#--------------------------------------
# file types to give credit for... updated
#
set cred_types {{[rs01][0-9][0-9]} m1v m2v mpg mp3 avi zip rar}

#--------------------------------------
# script to call when id3 info exists, and cookie args 
# eg. '/bin/boo %dir%'
#
# cookies: %dir%  %title%  %genre%  %artist%  %year%  %album%
#
set id3script ""

####################################################### begin code.
set debug 0

source /bin/ss-glclib.tcl
if {![info exists glcver] || $glcver != 0.1} {puts "ERROR loading glclib, missing or incorrect version\n" ; exit 0}
set user $env(USER)
set group $env(GROUP)
set wbuf "" ; set nfo ""
set pcmd(ss:x5) ""
set pcred(ss:x5) ""
set pgrp(ss:x5) ""

proc uuf {user size files section ratio} { 
  global userpath tmp statup
  set wbuf ""
  set fout [ardw "$userpath/$user.lock" [pid]]
  if {$fout == "e"} {puts "ERROR writing userfile lock" ;  return}
  set sect1 [expr ($section * 3) + 1]
  incr section
  set fin [split [ardw "$userpath/$user"] \n]
  foreach line $fin {
        if [string equal [lindex $line 0] "ALLUP"] {
		if {!$statup} {continue}
		set cnt 0 ; set value ""
		foreach {cnt value} [lrange $line $sect1 [expr $sect1 + 1]] {}
		incr cnt $files
		if {$value != ""} {
			set value [expr $value / 1000]
			set value [expr $size + $value] ; append value "000"
			set line [lreplace $line $sect1 [expr $sect1 + 1] $cnt $value]
		} else {
			append line " $cnt $value 0"
		}
        } elseif [string equal [lindex $line 0] "CREDITS"] {
		set value [lindex $line $section]
		if {$value != ""} {
			set value [expr $value / 1000]
			set value [expr ($ratio * $size) + $value] ; append value "000"
			set line [lreplace $line $section $section $value]
		} else {
			append line " $value"
		}
        }
        lappend wbuf $line
  }
  set fout [ardw "$userpath/$user" $wbuf]
  if {$fout == "e"} {puts "ERROR updating userfile with credits"} else {puts ".. $size -> $user"}
  catch {file delete -force -- "$userpath/$user.lock"} 
  return 0
}

proc cfgloader {lines} {
  global pcmd pcred pgrp tcfg pdate comm debug
  set cmod 9 ; set comm ""
  if {$lines == "e"} {puts "ERROR failed to load configuration ..\nmake sure $tcfg exists!\n" ; return -code return}
  foreach line [split $lines \n] {
	set line [string trim $line]
	if {$line == ""} {continue
	} elseif [string equal -length 1 "#" $line] {lappend comm $line ; continue
	} elseif [string equal {[COMMANDS]} $line] {set cmod 1 ; continue
	} elseif [string equal {[GROUPS]} $line] {set cmod 2 ; continue
	} elseif [string equal {[CREDITING]} $line] {set cmod 3 ; continue
	} 
	switch $cmod {
		1 {	set dflag 0
			foreach {cmd pdest dflag} $line {break}
			set pcmd($cmd) $pdest
			set pdate($cmd) $dflag
		}
		2 {set pgrp([lindex $line 0]) [lindex $line 1]}
		3 {set pcred([lindex $line 0]) [lrange $line 1 2]}
	}
  }
  unset pcmd(ss:x5)
  unset pgrp(ss:x5)
  unset pcred(ss:x5)
}

proc cfgwriter {} {
  global pcmd pcred pgrp comm tcfg pdate
  lappend comm {} {[COMMANDS]}
  foreach {item val} [array get pcmd] {lappend comm "$item \"$val\" $pdate($item)"}
  lappend comm {} {[GROUPS]}
  foreach {item val} [array get pgrp] {lappend comm "$item \"$val\""}
  lappend comm {} {[CREDITING]}
  foreach {item val} [array get pcred] {lappend comm "$item $val"}
  lappend comm {}
  catch {file delete -force $tcfg}
  if {[ardw $tcfg $comm] == "e"} {puts "ERROR saving updated configuration failed!\n"}
}

proc id3parse {file} {
  global id3script
  set rval "" ; set title "" ; set artist "" ; set album "" ; set year "" ; set comment "" ;  set genre 12
set genres {
 Blues			{Classic Rock}		Country			Dance
 Disco			Funk			Grunge			Hip-Hop
 Jazz			Metal			{New Age}		Oldies
 Other			Pop			R&B			Rap
 Reggae			Rock			Techno			Industrial
 Alternative		Ska			{Death Metal}		Pranks
 Soundtrack		Euro-Techno		Ambient			Trip-Hop
 Vocal			Jazz+Funk		Fusion			Trance
 Classical		Instrumental		Acid			House
 Game			{Sound Clip}		Gospel			Noise
 AlternRock		Bass			Soul			Punk
 Space			Meditative		{Instrumental Pop}	{Instrumental Rock}
 Ethnic			Gothic			Darkwave		Techno-Industrial
 Electronic		Pop-Folk		Eurodance		Dream
 {Southern Rock}	Comedy			Cult			Gangsta
 {Top 40}		{Christian Rap}		Pop/Funk		Jungle
 {Native American}	Cabaret			{New Wave}		Psychadelic
 Rave			Showtunes		Trailer			Lo-Fi
 Tribal			{Acid Punk}		{Acid Jazz}		Polka
 Retro			Musical			{Rock & Roll}		{Hard Rock}
 Folk			Folk-Rock		{National Folk}		Swing
 {Fast Fusion}		Bebob			Latin			Revival
 Celtic			Bluegrass		Avantgarde		{Gothic Rock}
 {Progressive Rock}	{Psychedelic Rock}	{Symphonic Rock}	{Slow Rock}
 {Big Band}		Chorus			{Easy Listening}	Acoustic
 Humour			Speech			Chanson			Opera
 {Chamber Music}	Sonata			Symphony		{Booty Brass}
 Primus			{Porn Groove}		Satire			{Slow Jam}
 Club			Tango			Samba			Folklore
 Ballad			{Power Ballad}		{Rhytmic Soul}		Freestyle
 Duet			{Punk Rock}		{Drum Solo}		{A Capela}
 Euro-House		{Dance Hall}
}
  if {![file readable $file] || [catch {open $file r} of]} {set wbuf "" ; return "e"}
  fconfigure $of -encoding binary -translation binary
  set fsize [expr [file size $file] - 128]
  if {$fsize < 1} {return "e"}
  seek $of $fsize
  set tag [read $of 128]
  binary scan $tag A3A29A1A30A30A4A28ccc id title boo artist album year comment zero track genre
  set genre [lindex $genres $genre]

  lappend rval $title $artist $album $genre $year
  set id3script [string map "%title% \{$title\} %artist% \{$artist\} %album% \{$album\} %genre% \{$genre\} %year% $year" $id3script]
  return $rval
}

proc nose {pinf {text "boo?"}} {
  global mvs
  if {[string is integer $pinf] && $pinf < 1} {puts $text ; set mvs 0 ; return -code return}
  if {$pinf == ""} {puts $text ; set mvs 0 ; return -code return}
}

proc main {argvt} {
  global user group userpath userup cred_types pcred pgrp pcmd pdate tcfg mvs nfo debug mainlog dupelog id3script
  puts "-\[Sitestat-PRE\]-----------------------"
  cfgloader [ardw $tcfg]
  set stder "That target command is not defined..\ntry 'site pre help' to view available targets\n"
  set action "help" ; set type "help" ; set dir "" ; set val "" ; set extra 0
  foreach {action type dir val} $argvt {break}
  set action [string tolower $action]
  if {![string equal addpath $action] && ![string equal delpath $action]} {
    set type [string tolower $type]
  }
  set pinfo [lindex [array get pcmd $type] 1]
  set pwd [pwd] ; append pwd /
  if {$debug} {
    puts "debug = on"
    puts "action = $action"
    puts "type = $type"
    puts "dir = $dir"
    puts "val = $val"
    puts "pinfo = $pinfo"
    puts "grps = [array get pgrp($type)]"
    puts "cmds = [array get pcmd($type)]"
  }
  
  if ![string equal "pre" $action] {set mvs 0}
  switch -- $action {
    view {puts [ardw $tcfg]}
    pre {
	if [string equal "help" $type] {nose 0 "Syntax: site pre <target> <dir>\n\nValid Pre targets:\n-> [array names pcmd]\n"}
	nose $pinfo $stder
	set pregroup [predirchk $pwd]
	nose $pregroup "This is not a designated pre area\n"
	if {[lsearch -exact $pgrp($type) $pregroup] == -1} {nose 0 "You are not permitted to pre there\n"}
	if {![file isdirectory $dir] || $dir == ""} {nose {} "The specified dir doesn't appear to exist\n"}
	cd $dir
	set cdz [glob -nocomplain -type d \[cC\]\[dD\]\[0-9\]]
	append cdz " " [glob -nocomplain -type d \[cC\]\[dD\]\[0-9\]\[0-9\]]
	set cdz [llength $cdz]
	set tfsize 0 ; set fcnt 0 ; set filez ""
	append filez [glob -nocomplain -type f *] " "
	set dirz [glob -nocomplain -type d *]
	foreach dire $dirz {
		if {$dire == ".." || $dire == "."} {continue}
		append filez [glob -nocomplain -type f $dire/*] " "
	}
	set carray($user) 0
	set uidtab [uidr]
	foreach file $filez {
		set go 1
		set ext [string tolower [string range $file end-2 end]]
		if ![file readable $file] {continue}
		foreach ctype $cred_types {
			if [string match $ctype $ext] {set go 0 ; break}
		}
		if {$go} {continue}
		incr fcnt
		set fsize [expr [file size $file] / 1048576]
		incr tfsize $fsize
		if {$userup != 1} {continue}
		set owner [file attributes $file -owner]
		if [string is integer $owner] {
			set pas [lsearch $uidtab $owner] ; incr pas
			if {$pas != 0} {set owner [string trim [lindex $uidtab $pas] +]}
		}
		if [catch {incr carray($owner.0) $fsize}] {set carray($owner.0) $fsize}
		if [catch {incr carray($owner.1)}] {set carray($owner.1) 1}
	}
	set id3c [glob -nocomplain -type f "*.\[mM\]\[pP\]3"]
##	append id3c " " [glob -nocomplain -type f "*.\[mM\]\[12\]\[vV\]"]
##	append id3c " " [glob -nocomplain -type f "*.\[mM\]\[pP\]\[gG\]"]
	set id3 0
	if ![string equal "" $id3c] {
		set id3c [id3parse [lindex $id3c 0]]
		if [string equal "e" $id3c] {break}
		set id3 1
	}
	if {$id3} {set pfx "PRE+ID3:"} else {set pfx "PRE:"}
	cd ..
	set tss [clock seconds]
	set ddir $pinfo
	if {$pdate($type) == 1} {
		append ddir "[clock format $tss -format %m%d]/"
	} elseif {$pdate($type) == 2} {
		append ddir "week[clock format $tss -format %U]/"
	}
	append ddir $dir
	puts "Files: $fcnt + Size: [set tfsize]MB + CDs: $cdz"
	puts "Transferring $dir -> $ddir..."
	if [file exists $ddir] {
		puts "ERROR moving/copying dir:\n-> a file or dir exists by that name\n" ; return 1
	}
	if [catch {file rename -force -- $dir $ddir}] {
		catch {
			if [catch {file mkdir $ddir}] {puts "ERROR moving/copying dir:\n-> unable to create destination\n" ; return 1}
			set gowner [file attributes $dir -owner]
			set ggroup [file attributes $dir -group]
			file attributes $ddir -owner $gowner -group $ggroup
		}
		if [catch {file copy -force -- $dir $ddir} msg] {
			puts "ERROR moving/copying dir:"
			catch {file delete -force $ddir}
			catch {file delete -force $dir}
			puts "-> $msg\n" ; return 1
		}
		foreach filed [glob -nocomplain $ddir] {
			file attributes $filed -owner $gowner -group $ggroup
		}
	}
	puts "PRE Completed in [expr [clock seconds] - $tss] seconds... thank you for the support"
	file mtime $ddir $tss
	set ts [clock format $tss -format "%a %b %d %T %Y"]
	set of [open $mainlog a]
	set lw "$ts $pfx \"$ddir\" \"$pregroup\" \"$user\" \"$group\" $fcnt $tfsize $cdz"
	if {$id3} {
		append lw " \"$id3c\""
		if ![string equal "" $id3script] {catch {exec [string map "%dir% \{$ddir\}" $id3script]}}
	}
	puts $of $lw
	close $of
	set of [open $dupelog a]
	lappend dirz $dir
	set tss [clock format $tss -format "%m%d%y"]
	foreach dir $dirz {puts $of "$tss $dir"}
	close $of
	if {$userup == 0 || $fcnt == 0 || $fsize == 0} {
	} else {
		puts "Updating userfile(s)... "
		set section [lindex $pcred($type) 0]
		set ratio [lindex $pcred($type) 1]
		if {$userup == 1} {
			foreach {user fsize} [array get carray *.0] {
				set user [string range $user 0 end-2]
				if [file exists "$userpath/$user.lock"] {after 1000 {set rde 1} ; vwait rde}
				uuf $user $fsize $carray($user.1) $section $ratio
			}
		} else {
			if [file exists "$userpath/$user.lock"] {after 1000 {set rde 1} ; vwait rde}
			uuf $user $fsize $fcnt $section $ratio
		}
	}
	cd $ddir
	set nfos [glob -nocomplain -type f *.\[Nn\]\[Ff\]\[Oo\]]
	set stamp 0
	switch [llength $nfos] {
		0 {set mvs 0}
		1 {set nfo $nfos}
		default {
		  foreach file $nfos {
		    if {[file mtime $file] < $stamp} {continue}
		    set stamp [file mtime $file]
		    set nfo $file	
		  }
		}
	}
    }
    section {
	nose $pinfo $stder
	if ![string is integer $dir] {
		puts "Invalid credit section number, must be an integer\n(note: sections start at 0)\n"
		return
	}
	if ![info exists pcred($type)] {set pcred($type) "0 0"}
	set oldval $pcred($type)
	set pcred($type) "$dir [lindex $oldval 1]"
	puts "Pre target '$type' set to credit section '$dir'\n"
	cfgwriter
    }
    ratio {
	nose $pinfo $stder
	nose [string is integer $dir] "Invalid ratio specified, for '1:3' use '3'\n"
	if ![info exists pcred($type)] {set pcred($type) "0 0"}
	set oldval $pcred($type)
	set pcred($type) "[lindex $oldval 0] $dir"
	puts "Pre target '$type' set to ratio of '1:$dir'\n"
	cfgwriter
    }
    addpath {
	nose [string match "/site/*" $type] "Invalid path, it must start with /site/...\n"
	nose $dir "Missing group binding, specify the group name this path belongs to\n"
	if {[string index $type end] != "/"} {set path $type} else {set path [string range $type 0 end-1]}
	if ![fragw $path $dir] {
		puts "ERROR creating glftpd path settings\n" ; return
	}
	if ![preprep $path] {
		puts "ERROR creating private path\n" ; return
	}
	puts "Path '$path' added to pre dir list for group '$dir'\n"
    } 
    delpath {
	nose [string match "/site/*" $type] "Invalid path specified, must begin with /site/...\n"
	if {[string index $type end] == "/"} {set type [string range $type 0 end-1]}
	if ![fragd $type] {puts "ERROR updating private paths configuration!\n"}
	puts "Path '$type' removed pre paths list\n"
    } 
    addgrp {
	nose $pinfo $stder
	foreach item $pgrp($type) {
		if [string equal -nocase $item $dir] {puts "Group '$dir' is already allowed to pre to '$type'\n" ; return}
	}
	lappend pgrp($type) $dir
	puts "Group '$dir' added to allow list for preing to '$type'\n"
	cfgwriter
    }
    delgrp {
	nose $pinfo $stder ; set ofs 0 ; set go 0
	foreach item $pgrp($type) {
		if [string equal -nocase $item $dir] {set go 1 ; set pgrp($type) [lreplace $pgrp($type) $ofs $ofs] ; break}
		incr ofs
	}
	if {$go} {
		puts "Group '$dir' removed from allow list for pre target '$type'\n"
		cfgwriter
	} else {
		puts "Group '$dir' isn't in the target '$type' allow list\n"
	}
    }
    addcmd {
	if {$pinfo != ""} {puts "Pre target '$type' already defined, delete it first\n"; return}
	nose $dir "Pre target '$type' must have a destination path; eg. /site/DivX/\n"
	if {![string match "/*/" $dir] || ![file isdirectory $dir]} {
		puts "Pre destination path '$dir' isn't valid or doesn't exist\n"
		puts "(note: make sure the path has leading and trailing slashes)\n" ; return
	}
	set pcmd($type) $dir
	set pcred($type) "0 0"
	set pgrp($type) ""
	if {$val == 1} {
		set extra "<MMDD>/"
	} elseif {$val == 2} {
		set extra "<week##>/"
	} else {
		nose {} "Pre option for dated paths are, 0 for non-dated, 1 for daily (eg. 1225), 2 for weekly (eg. week33)\n"
	}
	set pdate($type) $val
	puts "Pre target '$type' destination set to '$dir$extra'\n"
	puts "(note: now set the credit section, ratio and add groups)\n"
	cfgwriter
    }
    delcmd {
	nose $pinfo $stder 
	catch {unset pcmd($type)}
	catch {unset pcred($type)}
	catch {unset pgrp($type)}
	puts "Pre target '$type' and related settings deleted\n"
	cfgwriter
    }
    default {
	puts "Syntax: site editpre <option> <target> ?<value>?\n"
	puts "options: addcmd delcmd addgrp delgrp section ratio"
	puts "targets: [array names pcmd]\n"
	puts "example usages:"
	puts "  site editpre addcmd mp3 /site/mp3s/ 1"
	puts "  site editpre addpath /site/private/BIG/ BIG-ISO"
	puts "  site editpre addgrp mp3 TRONIQ"
	puts "  site editpre delcmd divx"
	puts "  site editpre delgrp vcd CENTROOPER"
	puts "  site editpre section vcd 6"
	puts "  site editpre ratio 0day 3\n"
	puts "show configuration -> site editpre view\n"
    }
  }
}


main $argv
if {$mvs == 1} {
 puts "--------------------------------------------"
 puts "checking for iMDb data..."
 set argv $nfo
 source /bin/movie2.tcl
 puts "--------------------------------------------"
}
